#include "itext.h"
#include "itext_file.h"
#include "itext_f2.h"
#include "xtext.h"
#include "xjp.h"

#include "xtext_1.h"

#include "pthread.h"

struct PICT{
	char *s;
	int no;
	struct PICT *n;
};

static struct PICT *ph = NULL;
static struct PICT *pcs = NULL;
static int CHU_NO = 0;
static int RUBY_YPOS = 0;


static pthread_t pth;
static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

extern xtext_conf_t xconf;

static int XtextDrawKunString(		Display *disp,
					my_window_t *win,
					char_data_t *char_d,
					ITEXT_T *it,
					int x,
					int y		)
{
	ITEXTR_T *kt;
	int cx,cy;
	unsigned char c1,c2;
	unsigned char tmp[3];

	if(it == NULL || char_d == NULL) return -1;

	cx = x - (char_d->charSize_r / 3);
	cy = y + char_d->charSize_r + 1;

	for( kt = it->kun_head; kt != NULL; kt = kt->next){
		c1 = kt->kun->c1;
		c2 = kt->kun->c2;


		if( c1 == AMARK && c2 == SHARP ) continue;
		if( c1 == AMARK && c2 == EKAKO ) continue;

		if( c1 == AMARK && c2 == SKAKO ){
			int xx;

			xx = char_d->charSize;
			if(ChFlag(it->flag,_SML)) 
				xx = char_d->charSize_s;

			cx = (x + xx) - (char_d->charSize_r); 
		       	continue;
		}
		tmp[0] = c1; 
		tmp[1] = c2; 
		tmp[2] = 0;

		if(win->anti){
#if HAVE_LIBXFT
			char utf8[32];

			euc2utf8_c(XJP_CONV,utf8,(char *)tmp,2);
			XftDrawStringUtf8(win->draw,&(win->fg_color),win->font_rubi,cx,cy,(FcChar8 *)utf8,strlen(utf8));

#else

			XmbDrawString(disp,win->pix0,win->rubi_fs,win->gc,cx,cy,(char *)tmp,strlen((char *)tmp));
			
#endif
		}else{

			XmbDrawString(disp,win->pix0,win->rubi_fs,win->gc,cx,cy,(char *)tmp,strlen((char *)tmp));

		}

		cy += (char_d->charSize_r + 1);
	}
	return 0;
}
static int XtextDrawString(	Display *disp,
				my_window_t *win,
				char_data_t *char_d,
				ITEXT_T *it,
				int x,
				int y		)
{
	GC gc;
	XFontSet fs;
	unsigned char text[3];
	int xpad,ypad;
	int charSize,re;

	text[0] = it->c1; text[1] = it->c2; text[2] = '\0';
	gc = win->gc;
	fs = win->fs;
	charSize = char_d->charSize;

	if(ChFlag(it->flag,_CHU)) gc = win->rubi_gc;
	if(ChFlag(it->flag,_SML)){
		   	fs = win->fs_s;
			charSize = char_d->charSize_s;
	}

	GetOffset(it->off,&xpad,&ypad);
	XmbDrawString(disp,win->pix0,fs,gc,x+xpad,y+ypad,(char *)text,strlen((char *)text));
	re = y + ypad;

	if(ChFlag(it->flag,_KUN))
		XtextDrawKunString(disp,win,char_d,it,x+xpad,y+ypad);

	return re - charSize;
}
static int XtextDrawAntiString(	Display *disp,
				my_window_t *win,
				char_data_t *char_d,
				ITEXT_T *it,
				int x,
				int y		)
{
	int re = 0;
	int charSize = 0;

#if HAVE_LIBXFT

	XftFont *font;
	XftColor color;
	XGlyphInfo gi;
	int xoff = 0;
	int yoff = 0;
	int xpad,ypad;
	char utf8[32];
	unsigned char temp[3];

	temp[0] = it->c1; temp[1] = it->c2; temp[2] = '\0';
	euc2utf8_c(XJP_CONV,utf8,(char *)temp,3);

	font = win->font;
	color = win->fg_color;

	charSize = char_d->charSize;
	GetOffset(it->off,&xpad,&ypad);

	if(ChFlag(it->flag,_CHU)) color = win->rubi_color;
	if(ChFlag(it->flag,_FND)) color = win->find_color;
	if(ChFlag(it->flag,_SML)){
		font = win->font_s; 
		charSize = char_d->charSize_s; 
	}

	XftTextExtentsUtf8(disp,font,(FcChar8 *)utf8,strlen(utf8),&gi);
	xoff = (charSize - gi.xOff)/2; 
	yoff = 2;
	XftDrawStringUtf8(win->draw,&color,font,x+xoff+xpad,y+yoff+ypad,(FcChar8 *)utf8,strlen(utf8));

	re = y + yoff + ypad;
	
	if(ChFlag(it->flag,_KUN))
		XtextDrawKunString(disp,win,char_d,it,x+xoff+xpad,y+yoff+ypad);
	
#else

	return re = XtextDrawString(disp,win,char_d,it,x,y);

#endif

	return re - charSize;
}
static XImage *xtext_get_mask_image(	Display *disp,
					Window win,
					int x0,int y0,
					int w0,int h0,
					XImage *image,
					int x1,int y1,
					int w1,int h1,
					unsigned long bg_pixel
								)
{
	XImage *wim;
	unsigned long pixel;
	int x,y;


	if((wim = XGetImage(disp,win,x0,y0,w0,h0,AllPlanes,XYPixmap)) == NULL){
		fprintf(stderr,"Error: XGetImage() xtext_f2.c\n");

		return NULL;
	}

	for(x = 0; x < w1; x++){
		for(y = 0; y < h1; y++){

			pixel = XGetPixel(image,x,y);

			if(pixel != bg_pixel)
				XPutPixel(wim,x,y,pixel);
			/* test code 
			else
				XPutPixel(wim,x,y,0xffff);
			 --------- */
		}
	}

	return wim;
}
static int XtextDrawImageString(	Display *disp,
					my_window_t *win,
					char_data_t *char_d,
					ITEXT_T *it,
					int x0,
					int y0			)
{
	XImage *image_str = NULL; 
	XImage *simg = NULL;
	/*
	XGCValues gval;
	unsigned long pixel;
	int x1,y1;
	*/
	unsigned long bg;
	int x,y,w,h,iw,ih;
	int charSize;
	int xoff,yoff;

	image_str = xtext_image_data_get(char_d->im_head,it);
	if(image_str == NULL) return -1;

	/*
	XGetGCValues(disp,win->bgc,GCForeground,&gval);
	bg = gval.foreground;
	*/
	bg = win->bg_pixel;

	charSize = char_d->charSize;
	if(ChFlag(it->flag,_SML)) 
		charSize = char_d->charSize_s;
	if(!ChFlag(it->flag,_EJC))
		charSize = char_d->charSize_e;

	/* offset */
	/*
	temp[0] = it->c1; temp[1] = it->c2; temp[2] = '\0';
	GetOffset(it->off,&xoff,&yoff);
	printf("%s : %d %d\n",temp,xoff,yoff);
	*/

	GetOffset(it->off,&xoff,&yoff);
	/*
	if(ChFlag(it->flag,_SML)){
		printf("x: %d y: %d\n",xoff,yoff);
	}
	*/
	x = x0 + xoff;
	y = (y0 - charSize + 1) + yoff;
	/* ----- */

	iw = w = charSize;
	ih = h = charSize;

	/*
	 goto normal_put_image; 
	 */

	if( (x + w >= win->width) || (x <= 0) ){
		 /* fprintf(stderr,"Over Width\n"); */
		goto normal_put_image;
	}
	if( (y + h >= win->height - charSize) || (y <= 0) ){
		/* fprintf(stderr,"Over Height\n"); */
		goto normal_put_image;
	}

	if((simg = xtext_get_mask_image(disp,win->pix0,x,y,w,h,image_str,0,0,iw,ih,bg)) == NULL)
		goto normal_put_image;

	XPutImage(disp,win->pix0,win->gc,simg,0,0,x,y,simg->width,simg->height);
	XDestroyImage(simg);
	/*
	for(x1 = 0; x1 < iw; x1++){
		for(y1 = 0; y1 < ih; y1++){

			pixel = XGetPixel(image_str,x1,y1);
			if( pixel != bg )
				XPutPixel(win->pix0,x+x1,y+y1,pixel);
		}
	}
	*/


	return y;

normal_put_image:

	XPutImage(disp,win->pix0,win->gc,image_str,0,0,x,y,iw,ih);

	return y;
}
static int xtextDrawPicture(	Display *disp,
				my_window_t *win,
				char_data_t *char_d,
				struct PICT *ph		)
{
	struct PICT *p;
	mypic_t *mp;
	int i;

	if( ph == NULL ) return 0;

	pthread_mutex_init(&mutex,NULL);

	for( p = ph,i = 0; p != NULL; p = p->n,i++ ){

		if( (mp = (mypic_t *)malloc(sizeof(mypic_t))) != NULL){

			mp->path = p->s;
			mp->no = p->no;
			mp->m = win;
			mp->mutex = &mutex;

			if( win->anti ){
#if HAVE_LIBXFT
				euc2utf8_c(XJP_CONV,mp->f,xconf.chuki_font,strlen(xconf.chuki_font));
#else
				xtext_create_fontset(xconf.chuki_font,mp->f,512,"16");
#endif
			}else{
				xtext_create_fontset(xconf.chuki_font,mp->f,512,"16");
			}

			pthread_create(&pth,NULL,xtextPicThreadFunc,mp);

		}
	}

	return 0;
}
static int XtextDrawChuString(	Display *disp,
				my_window_t *win,
				char_data_t *char_d,
				ITEXT_T *it,
				int x0,
				int y0	)
{
	XRectangle ink,log;
	char temp[8];
	int w,x,y;

	static chum_t *cts;
	chum_t *ct;

	if(it == NULL) return -1;
	if(it->chu_string == NULL) return -1;

	CHU_NO++;


	if( ChFlag(it->flag,_PIC) ){ 
		struct PICT *pc;

		if( (pc = (struct PICT *)malloc(sizeof(struct PICT))) != NULL){
			pc->s = it->pict_path;
			pc->no = CHU_NO;
			pc->n = NULL;

			if( ph == NULL ){
				ph = pc;
			}else{
				pcs->n = pc;
			}
			pcs = pc;
		}
	}


	if( (ct = (chum_t *)malloc(sizeof(chum_t))) != NULL){
		ct->s = it->chu_string;
		ct->n = CHU_NO;
		ct->next = NULL;

		if( char_d->chead == NULL ){
			char_d->chead = ct;
		}else{
			cts->next = ct;
		}
		cts = ct;
	}

	sprintf(temp,"(%d)",CHU_NO);

	if( win->anti ){
#if HAVE_LIBXFT
		char temp_u[64];
		XGlyphInfo gi;

		euc2utf8_c(XJP_CONV,temp_u,temp,strlen(temp));

		XftTextExtentsUtf8(disp,win->font_chu,(FcChar8 *)temp_u,strlen(temp_u),&gi);
		x = x0 + (char_d->charSize - gi.width)/2 + 1;
		y = y0 + (char_d->charSize_s + char_d->ypad);

		XftDrawStringUtf8(win->draw,&(win->s_color),win->font_chu,x,y,(FcChar8 *)temp_u,strlen(temp_u));

#else

		w = XmbTextExtents(win->chu_fs,temp,strlen(temp),&ink,&log);	
		x = x0 + (char_d->charSize - w)/2 + 1;
		y = y0 + (char_d->charSize_s + char_d->ypad);
		XmbDrawString(disp,win->pix0,win->chu_fs,win->sgc,x,y,temp,strlen(temp));
#endif

	}else{

		w = XmbTextExtents(win->chu_fs,temp,strlen(temp),&ink,&log);	
		x = x0 + (char_d->charSize - w)/2 + 1;
		y = y0 + (char_d->charSize_s + char_d->ypad);
		XmbDrawString(disp,win->pix0,win->chu_fs,win->sgc,x,y,temp,strlen(temp));

	}

	return 0;
}
static int XtextDrawRubiString(	Display *disp,
				my_window_t *win,
				char_data_t *char_d,
				ITEXT_T *it,
				int x0,
				int y0	)
{
	ITEXT_T *ib;
	ITEXTR_T *rb;
	unsigned char text[3];
	int k_len=0,r_len=0;
	int x,y,i;

	if(win->anti == 0){
		if(win->rubi_fs == NULL) return -1;
	}else{
#if HAVE_LIBXFT
		if(win->font_rubi == NULL) return -1;
#else
		if(win->rubi_fs == NULL) return -1;
#endif
	}
	if(ChFlag(it->flag,_RUB)){
		int m;

		GetFlag(it->flag,m);

		for(i=1;i<m;i++)
			k_len = k_len + (char_d->charSize + char_d->ypad);

	}else{
		for(ib=it;ib!=NULL;ib=ib->prev){
			if(!(ib->c1 >= 0xb0) && k_len != 0) break;

			k_len = k_len + (char_d->charSize + char_d->ypad);
		}
	}
	for(rb=it->rubi_head;rb!=NULL;rb=rb->next)
		r_len = r_len + char_d->charSize_r;

	if(k_len >= r_len)
		y = y0;
	else
		if((y = y0 + ((r_len - k_len)/2)) > win->height-2) y = y0;


	if(y - r_len < 0) y = r_len + char_d->charSize_r;
	if(y - r_len < RUBY_YPOS) y = RUBY_YPOS + r_len + char_d->charSize_r;
	RUBY_YPOS = y;
	x = x0 + (char_d->charSize - 2) + char_d->r_off;

	for(rb=it->rubi_tail;rb!=NULL;rb=rb->prev){
		text[0] = rb->c1; text[1] = rb->c2; text[2] = '\0';
		if(win->anti == 0){

			XmbDrawString(disp,win->pix0,win->rubi_fs,win->rubi_gc,
								x,y,(char *)text,strlen((char *)text));

		}else{

#if HAVE_LIBXFT
			char utf8[12];

			euc2utf8_c(XJP_CONV,utf8,(char *)text,2);
			XftDrawStringUtf8(win->draw,&(win->rubi_color),win->font_rubi,
								x,y,(FcChar8 *)utf8,strlen(utf8));
#else
			XmbDrawString(disp,win->pix0,win->rubi_fs,win->rubi_gc,
								x,y,(char *)text,strlen((char *)text));
#endif
		}
		y = y - char_d->charSize_r;
	}
	return 0;
}
static int XtextDrawBoutenString(	Display *disp,
					my_window_t *win,
					char_data_t *char_d,
					int btype,
					int x0,
					int y0			)
{
	int x,y,x1,y1;

	if(win == NULL) return -1;
	if(char_d == NULL) return -1;

	x = x0 + char_d->charSize;
	x1 = x + char_d->charSize_r;
	y = y0 - char_d->charSize + 4;
	y1 = y0;

	xtextDrawArc(disp,win,x,y,x1,y1,btype);

	return 0;
}
static int XtextDrawBousenString(	Display *disp,
					my_window_t *win,
					char_data_t *char_d,
					int btype,
					int x0,
					int y0			)
{
	int x,y,x1,y1;

	if(win == NULL) return -1;
	if(char_d == NULL) return -1;

	x = x0 + char_d->charSize + 1; x1 = x;
	y = y0 - char_d->charSize + 2; y1 = y0 + 2;

	xtextDrawLine(disp,win,x,y,x1,y1,btype);

	return 0;
}
static int _Draw_Page(Display *disp,my_window_t *win,char_data_t *char_d,page_data_t *char_p)
{
	char page_str[48];
	int width,height;
	int ftype;

	sprintf(page_str,"(%d/%d)",char_p->page_no,char_d->book->total_page);
	if( win->title_on )
		ftype = FTYPE_ENG;
	else
		ftype = FTYPE_RUB;

	xtext_get_string_size(disp,win,page_str,page_str,&width,&height,ftype);
	xtextDrawPageTitleString(disp,win,page_str,page_str,5,win->height-2,ftype);

	return 0;
}
static int _Draw_Title(Display *disp,my_window_t *win,char_data_t *char_d)
{
	char level_str[2][64],utf8[128],*p;
	int width,height;

	if(char_d->book->book_title != NULL && char_d->book->book_title_utf8 != NULL){

		xtext_get_string_size(disp,win,char_d->book->book_title,char_d->book->book_title_utf8,&width,&height,FTYPE_SMA);
		xtextDrawPageTitleString(disp,win,char_d->book->book_title,char_d->book->book_title_utf8,2,height,FTYPE_SMA);

	}

	switch(win->kinsc){
		case 0: strcpy(level_str[0],X_LEVEL0); break;
		case 1: strcpy(level_str[0],X_LEVEL1); break;
		case 2: strcpy(level_str[0],X_LEVEL2); break;
	}

	euc2utf8_c(XJP_CONV,level_str[1],level_str[0],strlen(level_str[0]));
	xtext_get_string_size(disp,win,level_str[0],level_str[1],&width,&height,FTYPE_SMA);
	xtextDrawPageTitleString(disp,win,level_str[0],level_str[1],win->width-width-5,height,FTYPE_SMA); 

	if(char_d->book->book == NULL) return 0;

	p = char_d->book->book;
	euc2utf8_c(XJP_CONV,utf8,p,strlen(p));
	xtext_get_string_size(disp,win,char_d->book->book,utf8,&width,&height,FTYPE_ENG);
	xtextDrawPageTitleString(disp,win,char_d->book->book,utf8,win->width-width-2,win->height-2,FTYPE_ENG);

	return 0;
}
/*
static int _Draw_Other(Display *disp,my_window_t *win,IOTHER_T *o_head)
{
	IOTHER_T *op;
	int i = 0,width,height;
	char *utf8,temp[64];

	if(o_head == NULL) return -1;

	for(op=o_head;op!=NULL;op=op->next)
		i++;

	sprintf(temp,"%s(%d)",XJP_MESSAGE[20],i);
	utf8 = (char *)euc2utf8(XJP_CONV,temp,strlen(temp));

	xtext_get_string_size(disp,win,temp,utf8,&width,&height,FTYPE_RUB);
	xtextDrawPageTitleString(disp,win,temp,utf8,(win->width-width)/2,height,FTYPE_RUB);

	free(utf8);

	return 0;
}
*/
static int _Draw_String(	Display *disp,
				my_window_t *win,
				char_data_t *char_d,
				ITEXT_T *it,
				int x,
				int y				)
{
	int re = 0;

	if(ChFlag(it->flag,_EJC)){
		int m;

		if((re = XtextDrawImageString(disp,win,char_d,it,x,y)) < 0){

			if(win->anti == 0)
				re = XtextDrawString(disp,win,char_d,it,x,y);
			else
				re = XtextDrawAntiString(disp,win,char_d,it,x,y);
		}

		if(win->page_on){

			if(ChFlag(it->flag,_TEN)){
				GetFlag(it->flag,m);
	       			XtextDrawBoutenString(disp,win,char_d,m,x,y);
			}
			if(ChFlag(it->flag,_SEN)){
				GetFlag(it->flag,m);
	       			XtextDrawBousenString(disp,win,char_d,m,x,y);
			}
			if(it->rubi_head != NULL) 
				XtextDrawRubiString(disp,win,char_d,it,x,y);
		}
	}else{

		re = XtextDrawImageString(disp,win,char_d,it,x,y);

	}

	if(it->chu_string != NULL) XtextDrawChuString(disp,win,char_d,it,x,y);

	return re;
}
static void xtextClearWindow(	Display *disp,my_window_t *win )
{
	if(disp == NULL || win == NULL)
		fprintf(stderr,"Error: xtextClearWidnow(xtext_f2.c)\n");

	XFillRectangle(	disp,win->pix0,win->bgc,0,0,win->width,win->height);
	XFillRectangle(	disp,win->pix1,win->bgc,0,0,win->width,win->height);

	return;
}
int xtext_Draw_Text(	Display *disp,
			my_window_t *win,
			ICONTENT_T *book,
			char_data_t *char_d,
			page_data_t *char_p	)
{
	static char *FileName;
	static int ScMax;

	IBUF_T *ib;
	ITEXT_T *it;

	int kin = 0, sc = 0, sc_max = 0,flag = 0;
	int x,y,h;
	int one_line,start_line,end_line;
	int S_R = 0,E_R = 0;

	if(	book == NULL 		|| 
		book->b_head == NULL 	|| 
		char_d == NULL 		|| 
		char_d->draw == NULL	){ 

		return -1;
	}

	if(char_p != NULL){
		sc_max = char_p->char_no; 
	}else{ 
		sc_max = 0;
	}

	
	if( FileName != book->filename || ScMax != sc_max ){

		FileName = book->filename;
		ScMax = sc_max;
	}

	xtextClearWindow(disp,win);

	CHU_NO = 0;
	RUBY_YPOS = 0;

	if(char_p != NULL){

		/*
		if(book->o_head != NULL) 
			_Draw_Other(disp,win,book->o_head);
			*/

		if(win->title_on) 
			_Draw_Title(disp,win,char_d); 

		if(win->page_on) 
			_Draw_Page(disp,win,char_d,char_p);

	}
	if( char_d->chead != NULL ){
		chu_data_free(char_d->chead);
		char_d->chead = NULL;
	}

	kin = win->kinsc;
	one_line = char_d->draw->one_line;
	start_line = char_d->draw->start_line;
	end_line = char_d->draw->end_line;

	x = char_d->draw->start_line;
	y = char_d->draw->start_low;

	for( ib = book->b_head; ib != NULL; ib = ib->next ){

		if( ib->i_head == NULL ){
			if( sc++ == sc_max ) flag++;
		}

		for(it = ib->i_head; it != NULL; it = it->next){

			if(sc++ >= sc_max) flag++;

			if( flag ){
				int yy,m;

				if( ChFlag(it->flag,_SML) ){

					GetFlag1(it->flag,m);

					S_R = char_d->draw->start_low + (char_d->charSize_s * m);
					E_R = char_d->draw->end_low - (char_d->charSize_s * 1);
				}else{
					S_R = char_d->draw->start_low;
					E_R = char_d->draw->end_low;
				}
				if( flag == 1 ) y = S_R;



				if(ChFlag(it->flag,_LST) || ChFlag(it->flag,_MID) || ChFlag(it->flag,_TOP)){ 
					y = xtext_get_low_pos(it,char_d,y);
				}

				yy = _Draw_String(disp,win,char_d,it,x,y);

				h = xtext_get_low_space(it,char_d,kin);

				if( (y += h) > E_R ){

					if(xtext_next_kuten(it,char_d,kin) == 0){

						if((x -= one_line) <= end_line) goto _draw_end;

						if(xtext_prev_kuten(it,char_d,kin) == 1){
							int xwidth,xwi;
							/* xprint(it); */

							xwidth = !ChFlag(it->flag,_SML) ? char_d->charSize:char_d->charSize_s;
							xwi = !win->title_on ? char_d->charSize_r:char_d->charSize_e;

							XCopyArea(	disp,
									win->pix1,
									win->pix0,
									win->bgc,
									x + one_line - (char_d->charSize_r / 3),
									yy,
									xwidth + char_d->xpad + char_d->r_off + char_d->charSize_r,
									win->height - yy - xwi,
									x + one_line - (char_d->charSize_r / 3),
					 				yy		 );

					       		it = it->prev;

						}
						y = S_R;
						RUBY_YPOS = 0;
					}
				}
			}
		}
		if(flag){
			if((x -= one_line) <= end_line ) 
				goto _draw_end;

			y = S_R;
			RUBY_YPOS = 0;
		}
	}

_draw_end:

	if( ph != NULL ){
		struct PICT *p,*pp;

		xtextDrawPicture(disp,win,char_d,ph);

		for( p = ph; p != NULL; p = pp ){
			pp = p->n;
			free(p);
		}
		ph = NULL;
	}

	if(char_p != NULL)
		return char_p->char_no;
	else
		return 0;
}
int xtext_Draw_Text_v(     	Display *disp,
                        	my_window_t *win,
                        	char *str[],
                        	int no,
                        	int xpos,
                        	int ypos		)                
{
        ITEXT_T *ihead = NULL, *itail = NULL;
        ICONTENT_T *line;
        char_data_t *char_d;
        int i,r,c,s;

	if(disp == NULL || win == NULL) return 0;
        win->change_font = 1;

	if((line = (ICONTENT_T *)content_link_alloc()) == NULL) return 0;
	if((char_d = (char_data_t *)char_data_alloc()) == NULL) return 0;

        char_d->xpos = xpos; 
	char_d->ypos = ypos;
        for(i = 0; i< no; i++){

		ihead = NULL; itail = NULL; 
		r = 0; c = 0; s = 0;

		itext_add(str[i],&ihead,&itail);
                ibuf_add(&(line->b_head),&(line->b_tail),ihead,itail,str[i],no);

        }

        xtext_Init_Text(disp,win,line,char_d);
        xtext_Draw_Text(disp,win,line,char_d,NULL);
        win->change_font = 1;

        ibuf_data_free(line);
	char_data_free(char_d);

        return 0;
}
int xtext_func(Display *disp,my_window_t *win,xtext_conf_t *xconf)
{
        char h[] = {	0x68,0x74,0x74,0x70,0x3a,0x2f,0x2f,\
			0x78,0x6a,0x70,0x2d,0x74,0x65,0x78,\
	     		0x74,0x2e,0x73,0x6f,0x75,0x72,0x63,\
			0x65,0x66,0x6f,0x72,0x67,0x65,0x2e,\
			0x6a,0x70,0x2f,0x0			};

        int x0,y0,width,height;
        int font_type;

        XClearWindow(disp,win->win);
	m_mes(disp,win);
        font_type = 3;
        xtext_get_string_size(disp,win,h,h,&width,&height,font_type);
        x0 = win->width - width - 5;
        y0 = win->height - 5;
        xtextDrawPageTitleString(disp,win,h,h,x0,y0,font_type);

        return 0;
}

